@@ -4,30 +4,39 @@
 #' futures in a container (e.g. list or environment) to be resolved while in
 #' the meanwhile retrieving values of already resolved futures.
 #' 
-#' @param x a list, an environment, or a list environment holding futures
-#' that should be resolved.  May also be a single \link{Future}.
+#' @param x A \link{Future} to be resolved, or a list, an environment, or a
+#' list environment of futures to be resolved.
 #' 
 #' @param idxs (optional) integer or logical index specifying the subset of
 #' elements to check.
 #' 
-#' @param result If TRUE, the results are retrieved, otherwise not.
-#' 
-#' @param value (DEPRECATED) Use argument `result` instead.
-#' 
 #' @param recursive A non-negative number specifying how deep of a recursion
 #' should be done.  If TRUE, an infinite recursion is used.  If FALSE or zero,
 #' no recursion is performed.
 #' 
+#' @param result (internal) If TRUE, the results are retrieved, otherwise not.
+#' 
+#' @param stdout (internal) If TRUE, captured standard output is relayed, otherwise note.
+#' 
+#' @param signal (internal) If TRUE, captured \link[base]{conditions} are relayed,
+#' otherwise not.
+#' 
+#' @param force (internal) If TRUE, captured standard output and captured
+#' \link[base]{conditions} already relayed is relayed again, otherwise not.
+#' 
 #' @param sleep Number of seconds to wait before checking if futures have been
 #' resolved since last time.
+#'
+#' @param progress (DEFUNCT) Defunct since future 1.14.0 to make room for
+#' other progress-update mechanisms that are in the works.
+#'
+#' @param value (DEPRECATED) Use argument `result` instead.
 #' 
-#' @param progress (DEPRECATED) If TRUE textual progress summary is outputted.  If a
-#' function, the it is called as \code{progress(done, total)} every time a
-#' future is resolved.
-#' 
-#' @param \dots Not used
+#' @param \dots Not used.
 #'
 #' @return Returns \code{x} (regardless of subsetting or not).
+#' If \code{signal} is TRUE and one of the futures produces an error, then
+#' that error is produced.
 #'
 #' @details
 #' This function is resolves synchronously, i.e. it blocks until \code{x} and
@@ -38,13 +47,13 @@
 #' \code{resolve(futureOf(x))}.
 #'
 #' @export
-resolve <- function(x, idxs = NULL, result = FALSE, value = result, recursive = 0, sleep = 1.0, progress = getOption("future.progress", FALSE), ...) UseMethod("resolve")
+resolve <- function(x, idxs = NULL, recursive = 0, result = FALSE, stdout = FALSE, signal = FALSE, force = FALSE, sleep = 1.0, value = result, progress = FALSE, ...) UseMethod("resolve")
 
 #' @export
 resolve.default <- function(x, ...) x
 
 #' @export
-resolve.Future <- function(x, idxs = NULL, result = FALSE, value = result, recursive = 0, sleep = 0.1, progress = FALSE, ...) {
+resolve.Future <- function(x, idxs = NULL, recursive = 0, result = FALSE, stdout = FALSE, signal = FALSE, force = FALSE, sleep = 0.1, value = result, ...) {
   ## BACKWARD COMPATIBILITY
   if (value && missing(result)) {
 ##    .Deprecated(msg = "Argument 'value' of resolve() is deprecated. Use 'result' instead.")
@@ -59,6 +68,9 @@ resolve.Future <- function(x, idxs = NULL, result = FALSE, value = result, recur
   ## Nothing to do?
   if (recursive < 0) return(x)
 
+  relay <- (stdout || signal)
+  result <- result || relay
+
   ## Lazy future that is not yet launched?
   if (x$state == 'created') x <- run(x)
 
@@ -79,11 +91,18 @@ resolve.Future <- function(x, idxs = NULL, result = FALSE, value = result, recur
     }
     
     ## Recursively resolve result value?
-    value <- x$result$value
-    if (!is.atomic(value)) {
-      value <- resolve(value, result = TRUE, recursive = recursive - 1, sleep = sleep, progress = FALSE, ...)
-      msg <- sprintf("%s (and resolved itself)", msg)
+    if (recursive > 0) {
+      value <- x$result$value
+      if (!is.atomic(value)) {
+        resolve(value, recursive = recursive - 1, result = TRUE, stdout = stdout, signal = signal, sleep = sleep, ...)
+        msg <- sprintf("%s (and resolved itself)", msg)
+      }
+      value <- NULL  ## Not needed anymore
     }
+    result <- NULL     ## Not needed anymore
+
+    if (stdout) value(x, stdout = TRUE, signal = FALSE)
+    if (signal) signalConditions(x, force = FALSE)
   } else {
     msg <- sprintf("%s (result was not collected)", msg)
   }
@@ -95,7 +114,7 @@ resolve.Future <- function(x, idxs = NULL, result = FALSE, value = result, recur
 
 
 #' @export
-resolve.list <- function(x, idxs = NULL, result = FALSE, value = result, recursive = 0, sleep = 0.1, progress = getOption("future.progress", FALSE), ...) {
+resolve.list <- function(x, idxs = NULL, recursive = 0, result = FALSE, stdout = FALSE, signal = FALSE, force = FALSE, sleep = 0.1, value = result, progress = FALSE, ...) {
   ## BACKWARD COMPATIBILITY
   if (value && missing(result)) {
 ##    .Deprecated(msg = "Argument 'value' of resolve() is deprecated. Use 'result' instead.")
@@ -115,27 +134,15 @@ resolve.list <- function(x, idxs = NULL, result = FALSE, value = result, recursi
   ## Nothing to do?
   if (nx == 0) return(x)
 
+  relay <- (stdout || signal)
+  result <- result || relay
+
+  if (!identical(progress, FALSE)) {
+    .Defunct(msg = "Argument 'progress' of resolve() is defunct.")
+  }
+
   x0 <- x
 
-  hasProgress <- ((is.logical(progress) && progress) || is.function(progress))
-  if (hasProgress) {
-    .Deprecated(msg = "Argument 'progress' of resolve() has been deprecated.")
-  }
-  
-  ## Setup default progress function?
-  if (hasProgress && !is.function(progress)) {
-    progress <- function(done, total) {
-      msg <- sprintf("Progress: %.0f%% (%d/%d)", 100 * done / total, done, total)
-      if (done < total) {
-        bs <- paste(rep("\b", times = nchar(msg)), collapse = "")
-        message(paste(msg, bs, sep = ""), appendLF = FALSE)
-      } else {
-        message(msg)
-      }
-    } ## progress()
-  }
-
-
   ## Subset?
   if (!is.null(idxs)) {
     ## Nothing to do?
@@ -181,10 +188,8 @@ resolve.list <- function(x, idxs = NULL, result = FALSE, value = result, recursi
   total <- nx
   remaining <- seq_len(nx)
 
-  if (hasProgress) {
-    done0 <- done <- 0
-    progress(done, total)
-  }
+  ## Relay?
+  signalConditionsASAP <- make_signalConditionsASAP(nx, stdout = stdout, signal = signal, force = force, debug = debug)
 
   if (debug) {
     mdebugf(" length: %d", nx)
@@ -196,36 +201,45 @@ resolve.list <- function(x, idxs = NULL, result = FALSE, value = result, recursi
     for (ii in remaining) {
       obj <- x[[ii]]
 
-      if (!is.atomic(obj)) {
+      if (is.atomic(obj)) {
+        if (relay) signalConditionsASAP(obj, resignal = FALSE, pos = ii)
+      } else {
         ## If an unresolved future, move on to the next object
         ## so that future can be resolved in the asynchronously
         if (inherits(obj, "Future")) {
           ## Lazy future that is not yet launched?
           if (obj$state == 'created') obj <- run(obj)
           if (!resolved(obj)) next
+          if (debug) mdebugf("Future #%d", ii)
+          if (result) value(obj, stdout = FALSE, signal = FALSE)
         }
+	  
+        relay_ok <- relay && signalConditionsASAP(obj, resignal = FALSE, pos = ii)
 
         ## In all other cases, try to resolve
-        resolve(obj, result = result, recursive = recursive - 1, sleep = sleep, progress = FALSE, ...)
+        resolve(obj,
+                recursive = recursive - 1,
+                result = result,
+                stdout = stdout && relay_ok,
+                signal = signal && relay_ok,
+                sleep = sleep, ...)
       }
 
       ## Assume resolved at this point
       remaining <- setdiff(remaining, ii)
       if (debug) mdebugf(" length: %d (resolved future %s)", length(remaining), ii)
       stop_if_not(!anyNA(remaining))
-
-      if (hasProgress) {
-        done <- total - length(remaining)
-        progress(done, total)
-      }
     } # for (ii ...)
 
     ## Wait a bit before checking again
     if (length(remaining) > 0) Sys.sleep(sleep)
   } # while (...)
 
-  if (hasProgress && done != done0) progress(done, total)
-
+  if (relay || force) {
+    if (debug) mdebug("Relaying remaining futures")
+    signalConditionsASAP(resignal = FALSE, pos = 0L)
+  }
+  
   if (debug) mdebug("resolve() on list ... DONE")
 
   x0
@@ -233,7 +247,7 @@ resolve.list <- function(x, idxs = NULL, result = FALSE, value = result, recursi
 
 
 #' @export
-resolve.environment <- function(x, idxs = NULL, result = FALSE, value = result, recursive = 0, sleep = 0.1, progress = FALSE, ...) {
+resolve.environment <- function(x, idxs = NULL, recursive = 0, result = FALSE, stdout = FALSE, signal = FALSE, force = FALSE, sleep = 0.1, value = result, ...) {
   ## BACKWARD COMPATIBILITY
   if (value && missing(result)) {
 ##    .Deprecated(msg = "Argument 'value' of resolve() is deprecated. Use 'result' instead.")
@@ -281,6 +295,9 @@ resolve.environment <- function(x, idxs = NULL, result = FALSE, value = result,
   nx <- length(idxs)
   if (nx == 0) return(x)
 
+  relay <- (stdout || signal)
+  result <- result || relay
+
   debug <- getOption("future.debug", FALSE)
   if (debug) {
     mdebug("resolve() on environment ...")
@@ -291,30 +308,45 @@ resolve.environment <- function(x, idxs = NULL, result = FALSE, value = result,
   x0 <- x
   x <- futures(x)
   nx <- .length(x)
-  idxs <- ls(envir = x, all.names = TRUE)
-  stop_if_not(length(idxs) == nx)
+  names <- ls(envir = x, all.names = TRUE)
+  stop_if_not(length(names) == nx)
 
   ## Everything is considered non-resolved by default
-  remaining <- idxs
+  remaining <- seq_len(nx)
+  
+  ## Relay?
+  signalConditionsASAP <- make_signalConditionsASAP(nx, stdout = stdout, signal = signal, force = force, debug = debug)
 
   if (debug) mdebugf(" elements: [%d] %s", nx, hpaste(sQuote(idxs)))
 
   ## Resolve all elements
   while (length(remaining) > 0) {
     for (ii in remaining) {
-      obj <- x[[ii]]
+      name <- names[ii]
+      obj <- x[[name]]
 
-      if (!is.atomic(obj)) {
+      if (is.atomic(obj)) {
+        if (relay) signalConditionsASAP(obj, resignal = FALSE, pos = ii)
+      } else {
         ## If an unresolved future, move on to the next object
         ## so that future can be resolved in the asynchronously
         if (inherits(obj, "Future")) {
           ## Lazy future that is not yet launched?
           if (obj$state == 'created') obj <- run(obj)
           if (!resolved(obj)) next
+          if (debug) mdebugf("Future #%d", ii)
+          if (result) value(obj, stdout = FALSE, signal = FALSE)
         }
 
+        relay_ok <- relay && signalConditionsASAP(obj, resignal = FALSE, pos = ii)
+
         ## In all other cases, try to resolve
-        resolve(obj, result = result, recursive = recursive-1, sleep = sleep, progress = FALSE, ...)
+        resolve(obj,
+                recursive = recursive - 1,
+                result = result,
+                stdout = stdout && relay_ok,
+                signal = signal && relay_ok,
+                sleep = sleep, ...)
       }
 
       ## Assume resolved at this point
@@ -327,6 +359,11 @@ resolve.environment <- function(x, idxs = NULL, result = FALSE, value = result,
     if (length(remaining) > 0) Sys.sleep(sleep)
   } # while (...)
 
+  if (relay || force) {
+    if (debug) mdebug("Relaying remaining futures")
+    signalConditionsASAP(resignal = FALSE, pos = 0L)
+  }
+  
   if (debug) mdebug("resolve() on environment ... DONE")
 
   x0
@@ -334,7 +371,7 @@ resolve.environment <- function(x, idxs = NULL, result = FALSE, value = result,
 
 
 #' @export
-resolve.listenv <- function(x, idxs = NULL, result = FALSE, value = result, recursive = 0, sleep = 0.1, progress = FALSE, ...) {
+resolve.listenv <- function(x, idxs = NULL, recursive = 0, result = FALSE, stdout = FALSE, signal = FALSE, force = FALSE, sleep = 0.1, value = result, ...) {
   ## BACKWARD COMPATIBILITY
   if (value && missing(result)) {
 ##    .Deprecated(msg = "Argument 'value' of resolve() is deprecated. Use 'result' instead.")
@@ -391,6 +428,8 @@ resolve.listenv <- function(x, idxs = NULL, result = FALSE, value = result, recu
   nx <- length(idxs)
   if (nx == 0) return(x)
 
+  relay <- (stdout || signal)
+  result <- result || relay
 
   debug <- getOption("future.debug", FALSE)
   if (debug) {
@@ -406,6 +445,9 @@ resolve.listenv <- function(x, idxs = NULL, result = FALSE, value = result, recu
   ## Everything is considered non-resolved by default
   remaining <- seq_len(nx)
 
+  ## Relay?
+  signalConditionsASAP <- make_signalConditionsASAP(nx, stdout = stdout, signal = signal, force = force, debug = debug)
+
   if (debug) {
     mdebugf(" length: %d", nx)
     mdebugf(" elements: %s", hpaste(sQuote(names(x))))
@@ -416,17 +458,28 @@ resolve.listenv <- function(x, idxs = NULL, result = FALSE, value = result, recu
     for (ii in remaining) {
       obj <- x[[ii]]
 
-      if (!is.atomic(obj)) {
+      if (is.atomic(obj)) {
+        if (relay) signalConditionsASAP(obj, resignal = FALSE, pos = ii)
+      } else {
         ## If an unresolved future, move on to the next object
         ## so that future can be resolved in the asynchronously
         if (inherits(obj, "Future")) {
           ## Lazy future that is not yet launched?
           if (obj$state == 'created') obj <- run(obj)
           if (!resolved(obj)) next
+          if (debug) mdebugf("Future #%d", ii)
+          if (result) value(obj, stdout = FALSE, signal = FALSE)
         }
 
+        relay_ok <- relay && signalConditionsASAP(obj, resignal = FALSE, pos = ii)
+
         ## In all other cases, try to resolve
-        resolve(obj, result = result, recursive = recursive-1, sleep = sleep, progress = FALSE, ...)
+        resolve(obj,
+                recursive = recursive - 1,
+                result = result,
+                stdout = stdout && relay_ok,
+                signal = signal && relay_ok,
+                sleep = sleep, ...)
       }
 
       ## Assume resolved at this point
@@ -439,6 +492,11 @@ resolve.listenv <- function(x, idxs = NULL, result = FALSE, value = result, recu
     if (length(remaining) > 0) Sys.sleep(sleep)
   } # while (...)
 
+  if (relay || force) {
+    if (debug) mdebug("Relaying remaining futures")
+    signalConditionsASAP(resignal = FALSE, pos = 0L)
+  }
+
   if (debug) mdebug("resolve() on list environment ... DONE")
 
   x0
