@@ -2,7 +2,12 @@ source("incl/start.R")
 
 message("*** Futures - labels ...")
 
-for (strategy in supportedStrategies()) {
+strategies <- supportedStrategies()
+
+## Speed up CRAN checks: Skip on CRAN Windows 32-bit
+if (!fullTest && isWin32) strategies <- NULL
+
+for (strategy in strategies) {
   message(sprintf("- plan('%s') ...", strategy))
   plan(strategy)
 
